#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   abaqus2fistr.py
#
#       abaqus から fistr へメッシュ変換
#
#       abaqusType  node    fistrType
#       ---------------------------------------------
#       C3D4        4       341         四面体1次
#       C3D10       10      342         四面体2次
#       C3D8        8       361         六面体1次
#       C3D20       20      362         六面体2次
#       STRI35      3       (S341)      表面要素（四面体1次)
#       S6          6       (S342)      表面要素（四面体2次）
#       S4R5        4       (S361)      表面要素（六面体1次）
#       S8          8       (S362)      表面要素（六面体2次）
#
#
#   15/03/19    新規作成
#      03/27    mashDataを数値で読み込みに変更
#      03/29    SGROUPを設定
#

import os
import sys

ls=os.linesep           #行末コードを定義

abaName = ""
nodeList = []           #NODE
elementList = []        #ELEMENT
surfaceList = []
surGroupList = []       #SGROUP
elsetList = []
ndsetList = []          #NGROUP
nElementType = 0        #elementTypeの種類


#  printMsg
#    print help
def printMsg():
    msg = """
<abaqus2fistr.py>
abaqusフォーマットのmeshをFrontISTR用mesh(HECMWフォーマット）に変換する。

<使い方>
abaqus2fistr.py <inpファイル>

例：
abaqus2fistr.py name.inp
abaqus2fistr.py name
"""
    print msg


#
#  getNodeElementData
#  ------------------
#    必要なdataを取得
def getNodeElementData(lines):
    item = ""
    header = ""
    data = []
    n = 0
    while n < len(lines):
        line = lines[n]
        if line[0] == "*":
            if len(data) > 0:
                saveData(item, header, data)

            #nodeの取得
            if line[:len("*NODE")] == "*NODE":
                header = line
                item = "NODE"
                length = 4
                data = []
            #要素の取得
            elif line[:len("*ELEMENT")] == "*ELEMENT":
                typeName = getValName(line, "TYPE")
                #四面体1次
                if typeName == "C3D4":
                    header = setValName(line, "TYPE=341")
                    item = "ELEMENT"
                    length = 5
                    data = []
                #四面体2次
                elif typeName == "C3D10":
                    header = setValName(line, "TYPE=342")
                    item = "ELEMENT"
                    length = 11
                    data = []
                #六面体1次
                elif typeName == "C3D8":
                    header = setValName(line, "TYPE=361")
                    item = "ELEMENT"
                    length = 9
                    data = []
                #六面体2次
                elif typeName == "C3D20":
                    header = setValName(line, "TYPE=362")
                    item = "ELEMENT"
                    length = 21
                    data = []
                #表面要素（四面体1次）
                elif typeName == "STRI35":
                    header = setValName(line, "TYPE=S341")
                    item = "SURFACE"
                    length = 4
                    data = []
                #表面要素（四面体2次）
                elif typeName == "S6":
                    header = setValName(line, "TYPE=S342")
                    item = "SURFACE"
                    length = 7
                    data = []
                #表面要素（六面体1次）
                elif typeName == "S4R5":
                    header = setValName(line, "TYPE=S361")
                    item = "SURFACE"
                    length = 5
                    data = []
                #表面要素（六面体2次）
                elif typeName == "S8":
                    header = setValName(line, "TYPE=S362")
                    item = "SURFACE"
                    length = 9
                    data = []
                #以外の要素
                else:
                    item = ""
            #要素groupの取得
            elif line[:len("*ELSET")] == "*ELSET":
                header = line
                item = "ELSET"
                length = 1
                data = []
            #nodeGroupの取得
            elif line[:len("*NSET")] == "*NSET":
                header = line
                item = "NSET"
                length = 1
                data = []
            else:
                item = ""
            n += 1
        else:
            if item != "":
                [data, n] = getDataList(lines, n, item, length)
            else:
                n += 1
    if len(data) > 0:
        saveData(item, header, data)

#  getDataList
#    data部を取得
def getDataList(lines, n, item, length):
    data = []
    if item == "NODE":
        while n < len(lines) and lines[n][0] != "*":
            line = lines[n]
            words = line.split(",")
            if len(words) >= 4:
                a = [int(words[0]), float(words[1]), float(words[2]), float(words[3])]
            else:
                a = ["", "", "", ""]
            data.append(a)
            n += 1
    elif item == "ELEMENT" or item == "SURFACE":
        while n < len(lines) and lines[n][0] != "*":
            line = lines[n]
            words = line.split(ls)[0].split(",")
            for word in words:
                if len(word) > 0:
                    data.append(int(word))
            n += 1
        ans = []
        line = []
        p = 0
        for a in data:
            if p < length:
                line.append(a)
            else:
                ans.append(line)
                p = 0
                line = []
                line.append(a)
            p += 1
        ans.append(line)
        data = ans
    elif item == "ELSET" or item == "NSET":
        while n < len(lines) and lines[n][0] != "*":
            line = lines[n]
            words = line.split(ls)[0].split(",")
            for word in words:
                if len(word) > 0:
                    data.append(int(word))
            n += 1
    return [data, n]

#  saveData
#    読み込んだdataを取得する
def saveData(item, header, data):
    global nodeList, elementList, surfaceList, elsetList, ndsetList
    if item == "NODE":
        block = []
        num = str(len(data))
        header = "!NODE, NUM=" + num + ls
        block.append(header)
        block.append(data)
        nodeList.append(block)
        print "found NODE... num " + num
    elif item == "ELEMENT":
        block = []
        num = str(len(data))
        header = "!" + header[1:]
        header = deleteSp(header).split()[0]
        words = header.split(",")
        header = ", ".join(words) + ", NUM=" + num + ls
        block.append(header)
        block.append(data)
        elementList.append(block)
        name = getValName(header, "TYPE")
        print "found ELEMENT... TYPE " + name + " num " + num
    elif item == "SURFACE":
        block = []
        num = str(len(data))
        header = "!" + header[1:]
        header = deleteSp(header).split()[0]
        words = header.split(",")
        header = ", ".join(words) + ", NUM=" + num + ls
        block.append(header)
        block.append(data)
        surfaceList.append(block)
        name = getValName(header, "TYPE")
        print "found surfaceELEMENT... TYPE " + name + " num " + num
    elif item == "ELSET":
        block = []
        num = str(len(data))
        header = "!" + header[1:]
        header = deleteSp(header).split()[0]
        words = header.split(",")
        header = ", ".join(words) + ", NUM=" + num + ls
        block.append(header)
        block.append(data)
        elsetList.append(block)
        name = getValName(header, "ELSET")
        print "found ELSET... " + name + " num " + num
    elif item == "NSET":
        block = []
        num = str(len(data))
        header = "!" + header[1:]
        header = deleteSp(header).split()[0]
        words = header.split(",")
        header = ", ".join(words) + ", NUM=" + num + ls
        block.append(header)
        block.append(data)
        ndsetList.append(block)
        name = getValName(header, "NSET")
        print "found NSET... " + name + " num " + num


#
#  checkQuardElementList
#  ---------------------
#    2次メッシュのチェック
def checkQuardElementList():
    global elementList, nElementType
    #elementの種類の個数を取得
    nElementType = len(elementList)
    #2次メッシュのチェック
    i = 0
    while i < len(elementList):
        element = elementList[i]
        header = element[0]
        typeName = getValName(header, "TYPE")
        if typeName == "342":
            data = changeNodeOder342(element[1])
            elementList[i][1] = data
        elif typeName == "362":
            data = changeNodeOder362(element[1])
            elementList[i][1] = data
        i += 1
    #headerの修正
    remakeElementHeaderData()


#  changeNodeOder342
#    四面体2次メッシュの節点順を変更
def changeNodeOder342(data):
    i = 0
    while i < len(data):
        nums = data[i]
        newOder = [nums[0], nums[1], nums[2], nums[3], nums[4], nums[6], nums[7],
                nums[5], nums[8], nums[9], nums[10]]
        data[i] = newOder
        i += 1
    return data

#  changeNodeOder362
#    六面体2次メッシュの節点順を変更
def changeNodeOder362(data):
    i = 0
    while i < len(data):
        nums = data[i]
        newOder = [nums[0], nums[1], nums[5], nums[6], nums[2], nums[4], nums[8],
                nums[7], nums[3], nums[17], nums[13], nums[18], nums[9], nums[20],
                nums[15], nums[19], nums[11], nums[12], nums[16], nums[14], nums[10]]
        data[i] = newOder
        i += 1
    return data

#
#  remakeElementHeaderData
#    data整形
def remakeElementHeaderData():
    global elementList
    i = 0
    while i < len(elementList):
        header = elementList[i][0]
        typeName = getValName(header, "TYPE")
        num = getValName(header, "NUM")
        name = abaName.split(".")[0]
        header = "!ELEMENT, TYPE=" + typeName + ", PARTNAME=" + name + ", EGRP=" + name + ", NUM=" + num + ls
        elementList[i][0] = header
        i += 1

#
#  checkSurfaceElementGroup
#  ------------------------
#    surfaceのgroup分けをチェック
def checkSurfaceElementGroup():
    print "checking data..."
    global surGroupList, elsetList
    if len(elsetList) == 0:
        return
    if len(surfaceList) > 1:
        return
    print "creating SGROUP(surface)...",
    newSurElset = getSurfaceElset()                 #表面要素セットを取得
    newSurElset = addNodesSurElset(newSurElset)     #nodeNoを追加
    surfaceGroup = getElementFaceNo(newSurElset)    #elementを取得
    surGroupList = remakeHeaderSurGroup(surfaceGroup)   #header部を修正
    print

#  getSurfaceElset
#    表面要素セットを取得
def getSurfaceElset():
    newSurElset = []
    for elset in elsetList:
        if isSurfaceSet(elset) == True:
            newSurElset.append(elset)
    return newSurElset

#  isSurfaceSet
#    表面要素かどうかチェック
def isSurfaceSet(elset):
    ans = False
    maxElsetNo = max(elset[1])
    for surElement in surfaceList:
        maxSurNo = surElement[1][-1][0]
        if maxElsetNo <= maxSurNo:
            ans = True
            break
    return ans

#  addNodesSurElset
#    nodeNoを追加
def addNodesSurElset(elsets):
    newSurElset = []
    for elset in elsets:
        data = []
        for elNo in elset[1]:
            nodes = findSurfaceNodes(elNo)
            newNodes = [elNo] + nodes
            data.append(newNodes) 
        data = [elset[0]] + [data]
        newSurElset.append(data)
    return newSurElset

#  findSurfaceNodes
def findSurfaceNodes(elNo):
    ans = []
    flag = 0
    for surface in surfaceList:
        for nodeNo in surface[1]:
            if nodeNo[0] == elNo:
                flag = 1
                ans = nodeNo[1:]
                break
        if flag == 1:
            break
    return ans

#  getElementFaceNo
#    elementとfaceNo取得
def getElementFaceNo(elsets):
    #elementType取得
    elTypes = []
    for element in elementList:
        typeName = getValName(element[0], "TYPE")
        elTypes.append(typeName)
    #elementNo取得
    elementNoSet = []
    for elset in elsets:
        header = elset[0]
        name = getValName(header, "ELSET")
        print name,
        data = []
        for line in elset[1]:
            setNodes = line[1:]
            [elementNo, faceNo] = findElementNo(setNodes, elTypes)
            #data.append([elementNo] + [faceNo])
            data += [[elementNo] + [faceNo]]
        data.sort()
        #elementNoSet.append([header] + [data])
        elementNoSet += [[header] + [data]]
    return elementNoSet

#  findElementNo
def findElementNo(nodes, elTypes):
    elNo = 0
    flag = 0
    i = 0
    for element in elementList:
        for line in element[1]:
            if isInNodes(line[1:], nodes) == True:
                elNo = line[0]
                elType = elTypes[i]
                faceNo = getFaceNoOfElement(line[1:], nodes, elType)
                flag = 1
                break
        if flag == 1:
            break
        i += 1
    return [elNo, faceNo]

#  isInNodes
def isInNodes(line, nodes):
    flag = 0
    ans = False
    for node in nodes:
        for No in line:
            if node == No:
                flag += 1
                break
    if flag == len(nodes):
        ans = True
    return ans

#  remakeHeaderSurGroup
#    headerを書き換え
def remakeHeaderSurGroup(elsets):
    i = 0
    while i < len(elsets):
        elset = elsets[i]
        header = elset[0]
        groupName = getValName(header, "ELSET")
        num = getValName(header, "NUM")
        partName = abaName.split(".")[0]
        header = "!SGROUP, PARTNAME=" + name + ", SGRP=" + groupName + ", NUM=" + num + ls
        elsets[i][0] = header
        i += 1
    return elsets

#  getFaceNoOfElement
#    elementのfaceNo取得
def getFaceNoOfElement(elNodes, faceNodes, elType):
    faceNo = 0
    #四面体（1次、2次）要素の場合
    if elType == "341" or elType == "342":
        nodes = [1,2,3,4]
        i = 0
        newNodes = []
        while i < len(nodes):
            for faceNode in faceNodes:
                if elNodes[i] == faceNode:
                    newNodes.append(nodes[i])
                    break
            i += 1
        newNodes.sort()
        if newNodes == [1,2,3]:
            faceNo = 1
        elif newNodes == [1,2,4]:
            faceNo = 2
        elif newNodes == [2,3,4]:
            faceNo = 3
        elif newNodes == [1,3,4]:
            faceNo = 4
        else:
            faceNo = 0
    #六面体（1次、2次）要素の場合
    elif elType == "361" or elType == "362":
        nodes = [1,2,3,4,5,6,7,8]
        i = 0
        newNodes = []
        while i < len(nodes):
            for faceNode in faceNodes:
                if elNodes[i] == faceNode:
                    newNodes.append(nodes[i])
                    break
            i += 1
        newNodes.sort()
        if newNodes == [1,2,3,4]:
            faceNo = 1
        elif newNodes == [5,6,7,8]:
            faceNo = 2
        elif newNodes == [1,2,5,6]:
            faceNo = 3
        elif newNodes == [2,3,6,7]:
            faceNo = 4
        elif newNodes == [3,4,7,8]:
            faceNo = 5
        elif newNodes == [1,4,5,8]:
            faceNo = 6
        else:
            faceNo = 0
    return faceNo

#
#  checkElementGroup
#  -----------------
#    elementのgroup分けをチェック
def checkElementGroup():
    global elsetList, elementList, nElementType
    if len(elsetList) == 0:
        return

    print "creating EGROUP(element)...",
    #elementTypeの数は？
    if nElementType > 1:
        #複数のelementTypeがある場合
        newElGroup = []
        for elset in elsetList:
            if isElementSet(elset) == True:
                newData = createNewElGroup(elset)
                newElGroup.append(newData)
        elsetList = newElGroup
        print
        return

    #elementTypeが1種類の場合
    elSetList = []
    newElementList = []
    flag = 0
    for elset in elsetList:
        if isElementSet(elset) == True:
            flag = 1
            newData = createNewElementData(elset)
            newElementList.append(newData)
    if flag == 1:
        otherData = createOtherElementData(newElementList)
        if len(otherData[1]) > 0:
            newElementList.append(otherData)
        elementList = newElementList
    print

#  isElementSet
#    elementSetかどうか確認
def isElementSet(elset):
    ans = False
    maxElsetNo = max(elset[1])
    for element in elementList:
        minElNo = element[1][0][0]
        if maxElsetNo >= minElNo:
            ans = True
            break
    return ans

#  createNewElGroup
#    elsetを作成する
def createNewElGroup(elset):
    line = elset[0]
    name = getValName(line, "ELSET")
    partName = abaName.split(".")[0]
    num = getValName(line, "NUM")
    header = "!EGROUP, PARTNAME=" + partName + ", EGRP=" + name + ", NUM=" + num + ls
    elset[0] = header
    return elset

#  createNewElementData
#    elsetのelementDataを作成する
def createNewElementData(elset):
    line = elset[0]
    name = getValName(line, "ELSET")
    line = elementList[0][0]
    header = setValName(line, "EGRP="+name)
    print name,
    ans = []
    newElset = []
    for elNo in elset[1]:
        for element in elementList[0][1]:
            if element[0] == elNo:
                #newElset.append(element)
                newElset += [element]
                break
    num = str(len(newElset))
    header = setValName(header, "NUM="+num)
    ans.append(header)
    ans.append(newElset)
    return ans

#  createOtherElementData
#    otherElementを追加する
def createOtherElementData(newElementList):
    nEle = 0
    for element in newElementList:
        nEle += len(element[1])
    if nEle == len(elementList[0][1]):
        ans = ["", []]
        return ans

    print "other",
    otherData = []
    for elLine in elementList[0][1]:
        flag = 0
        elNo = elLine[0]
        for element in newElementList:
            for setLine in element[1]:
                setNo = setLine[0]
                if elNo == setNo:
                    flag = 1
                    break
            if flag == 1:
                break
        if flag == 0:
            otherData.append(elLine)
    header = elementList[0][0]
    header= setValName(header, "EGRP=other")
    num = str(len(otherData))
    header = setValName(header, "NUM="+num)
    ans = [header]
    ans.append(otherData)
    return ans

#
#  checkNodeGroup
#  --------------
#    nodeGroupをチェック
def checkNodeGroup():
    global ndsetList
    print "creating NGROUP(node)...",
    newNdset = []
    for ndSet in ndsetList:
        data = []
        header = ndSet[0]
        name = getValName(header, "NSET")
        print name,
        num = getValName(header, "NUM")
        partName = abaName.split(".")[0]
        header = "!NGROUP, PARTNAME=" + partName + ", NGRP=" + name + ", NUM=" + num + ls
        data.append(header)
        data.append(ndSet[1])
        data[1].sort()
        newNdset.append(data)
    ndsetList = newNdset
    print

#
#  witeData
#  --------
#    dataの書き込み
def writeData(fileName):
    print "writing..."
    cont = makeHeader()
    cont += makeNode()
    cont += makeElement()
    cont += makeElementGroup()
    cont += makeSurfaceGroup()
    cont += makeNodeGroup()
    cont += "!END"
    f=open(fileName, "w"); f.write(cont); f.close()

#  makeHeader
#    header
def makeHeader():
    cont = "!HEADER" + ls
    cont += "  genarated by abaqus2fistr.py" + ls
    return cont

#  makeNode
#    node
def makeNode():
    cont = ""
    for node in nodeList:
        cont += node[0]
        for nums in node[1]:
            strs = []
            for num in nums:
                strs.append(str(num))
            cont += ", ".join(strs) + ls
    return cont        

#  makeElement
#    element
def makeElement():
    cont = ""
    for element in elementList:
        header = element[0]
        name = getValName(header, "EGRP")
        cont += header
        for nums in element[1]:
            strs = []
            for num in nums:
                strs.append(str(num))
            cont += ", ".join(strs) + ls
        section = "!SECTION, TYPE=SOLID, EGRP=" + name + ls
        cont += section
    return cont

#  makeElementGroup
#    elementGroup
def makeElementGroup():
    cont = ""
    if nElementType == 1:
        return cont

    for elGroup in elsetList:
        if isElementSet(elGroup) == True:
            cont += elGroup[0]
            cont += makeStringFromData(elGroup[1], 8)
    return cont

#  makeSurfaceGroup
#    表面要素group
def makeSurfaceGroup():
    cont = ""
    for surGroup in surGroupList:
        cont += surGroup[0]
        for surSet in surGroup[1]:
            strs = []
            for num in surSet:
                strs.append(str(num))
            cont += ", ".join(strs) + ls
    return cont

#  makeNodeGroup
#    nodeGroup
def makeNodeGroup():
    cont = ""
    if len(ndsetList) == 0:
        return cont

    for ndset in ndsetList:
        cont += ndset[0]
        cont += makeStringFromData(ndset[1], 8)
    return cont

#  makeStringFromData
#    dataSetをstrに変換
def makeStringFromData(data, w):
    cont = ""
    p = 0
    while p+w < len(data):
        nums = data[p:p+w]
        strs = []
        for num in nums:
            strs.append(str(num))
        cont += ", ".join(strs) + "," + ls
        p += w
    nums = data[p:]
    strs = []
    for num in nums:
        strs.append(str(num))
    cont += ", ".join(strs) + ls
    return cont

#  getValName
#    valName(TYPEなど）を取得
def getValName(line, valName):
    words = line.split(",")
    name = ""
    for word in words:
        # 「TYPE=」などを検索
        if word.find(valName+"=") >= 0:
            name = word.split("=")[1]
            break
    newName = ""
    for chara in name:
        if chara > " ":
            newName += chara
    return newName

#  deleteSp
#    空白を削除
def deleteSp(line):
    newLine = ""
    for chara in line:
        if chara != " ":
            newLine += chara
    return newLine


#  setValName
#    変数の値をセット
def setValName(line, val):
    valNames = val.split("=")
    words = line.split(",")
    i = 0
    for word in words:
        name = (word.split()[0]).split("=")[0]
        if name == valNames[0]:
            words[i] = " " + val
            break
        i += 1 
    newLine = ",".join(words)
    if newLine[-1] != ls:
        newLine += ls
    return newLine


#
#  abaqus2fistr
#  ------------
#    convert abaqus to fistr
def abaqus2fistr(fileDir, abaName):
    fileName = fileDir + "/" + abaName + ".inp"
    f=open(fileName); lines=f.readlines(); f.close()
    #データ取得
    getNodeElementData(lines)
    #2次要素のチェック
    checkQuardElementList()
    #表面groupの取得
    checkSurfaceElementGroup()
    #要素groupの取得（単一要素の場合は、要素定義をgroup分けする）
    checkElementGroup()
    #nodeGroupの取得
    checkNodeGroup()
    #書き出し
    newName = fileDir + "/" + abaName + ".msh"
    writeData(newName)


if __name__=='__main__':
    abaNameDir = sys.argv[1]
    if abaNameDir[:len("-h")] == "-h" or abaNameDir[:len("--help")] == "--help":
        printMsg()
    else:
        fileDir = "/".join(abaNameDir.split("/")[:-1])
        if fileDir == "":
            fileDir = os.getcwd()
        abaName = abaNameDir.split("/")[-1]
        if abaName.split(".")[-1] == "inp":
            name = abaName[:-4]
        else:
            name = abaName
        abaqus2fistr(fileDir, name)

