#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  getFoamContents
#      Foam形式のデータを取得する
#
#       10/03/04    新規作成
#       11/11/05    cellZoneの取得変更（1行で記述されている場合を追加）
#       12/08/30    faceZoneが取得できていなかった為「getCellZonesContents」を修正
#       13/01/06    get1charaで「;」を文字以外に設定。
#          04/18    cellZoneNamesF, faceZoneNamesF:高速検索追加
#          05/13    cellZoneNamesDirを追加
#          06/10    getKeywordMem:「"」文字で囲まれた場合もkeywordとして取得
#                   get1lineMem追加（C++の行を取得）
#          07/10    getCurrMeshDirを追加
#          09/07    gzip形式の読込み、書込み追加
#          09/08    binary形式の読込み追加
#          09/13    getKeywordContentsMemを高速化（getMiddlePairMemFを使用）
#                   get1lineMemを高速化（getMiddlePairMemFを使用）
#          09/24    field読込をascii、dataに分離して読み込む方法に変更
#          09/25    meshのbinary化に伴う読込書込ルーチン追加
#          09/28    読込、書込を整形
#          10/04    バグ修正(実態がasciiFileの場合、formatがbinaryになっていた場合ｴﾗｰ
#          10/15    国際化のため修正
#          10/27    deleteNullLinesを追加
#          11/23    getC1lineMemを追加（C++の一行を取得）
#          12/21    stdout、stderrの設定（import logFileCreater）を追加
#          12/27    reconstFuseDataAtMaxLinesField:バグ修正.encode("utf-8")を削除
#       14/01/11    reconstFuseDataAtMaxLinesField:バグ修正（文字ｺｰﾄﾞの判定追加）
#          04/13    fuseAsciiData, reconstFuseDataAtMaxLinesMultiList, 
#                   reconstFuseDataAtMaxLinesField, getFuseDataAtMaxLinesAB:
#                   recObjとdataNoの取得方法修正（「**.**」のfield名が有るとｴﾗｰ）
#          07/26    controlDictを修正
#          08/10    get1lineMemを修正（other:「#xxxx yyyy」を追加）
#          08/25    get1lineMem:最終行がｺﾒﾝﾄ文で最終文字が"\n"でない場合、無限ﾙｰﾌﾟに入る
#          08/26    fuseAsciiData:受けた文字列を「str」に変換後、処理。
#          09/06    get1lineMem:line内に()や{}があった場合、行を判断できなかった。
#                   replace1lineMem（1行を入れ替える）を追加
#          09/07    getMiddlePairMemF:戻ってくるポインタpが1ヶ多かったので修正
#          09/21    get1c:修正（最終文字が認識できず）
#          11/10    fuseAsciiData:ascii部がunicodeの場合、ｴﾗｰ発生するのでstrに変換
#          12/12    get1lineMem:最初の文字が「;」の場合、lineContが空文になっていたので
#                   「;」を返す様に修正。
#                   getMiddlePairMemComm、getSmallParMemCommを追加（コメント文を
#                   そのまま残して取得）。
#                   get1lineMem:コメント文をそのまま残して取得する様に修正
#       16/05/09    readFieldFull:fieldの読み込みを"rb"に設定。
#                   
#

import os
import re
import glob
import string
import gzip
import struct
import pyTreeFoam
import getFoamContents

import logFileCreater; logFileCreater.log()


#------------------- file read write ------------------------------------------
#
#  readFile2000Bytes
#    fileを2000 byte読み込む（fileのheader部の読込）
def readFile2000Bytes(fileName):
    fileName = checkFileName(fileName)
    if fileName[-3:] == ".gz":
        #gzip形式でopen
        f = gzip.open(fileName)
    else:
        #非圧縮でopen
        f = open(fileName)
    #読込
    cont = f.read(2000)
    f.close()
    return cont

#
#  readFileFull
#    fileを全て読み込む（binary、asciiの区別なく）
def readFileFull(fileName):
    cont = readFieldFull(fileName)
    return cont

#
#  writeFileFull
#    fileを全てそのまま書き込む
def writeFileFull(fileName, cont):
    writeFieldFull(fileName, cont)
    return

#
#  readFileSeparate
#    fileをascii部、データ部に分けて読み込む
#    戻り値：[<ascii部>, <データ(ascii or binary)>]
def readFileSeparate(fileName):
    [ascii, data] = readMeshSeparate(fileName)
    return [ascii, data]

#
#  writeFileSeparate
#    分離されたfile内容をfileに書き込む
def writeFileSeparate(fileName, fileCont):
    writeMeshSeparate(fileName, fileCont)
    return

#
#  readFile
#    foamFileを読み込む
#    戻り値：asciiデータ
def readFile(fileName, nMaxLinesBin):
    partData = readField(fileName, nMaxLinesBin)
    return partData

#
#  writeFile
#    foamFileを書き込む
def writeFile(fileName, cont):
    writeField(fileName, cont)
    return

#
#  readField
#    fieldデータの読込
#    戻り値：asciiデータ
def readField(fileName, nMaxLinesBin):
    #separateデータの読込
    [ascii, data] = readFileSeparate(fileName)
    partData = getFuseDataAtMaxLinesAB(fileName, ascii, data, nMaxLinesBin)
    return partData

#
#  writeField
#    fieldデータの書き込み
def writeField(fileName, cont):
    allData = reconstFuseDataAtMaxLinesAB(fileName, cont)
    #全データの書き込み
    writeFieldFull(fileName, allData)
    return

#
#  readMeshSeparate
#    meshファイルの読込
def readMeshSeparate(fileName):
    cont = readFileFull(fileName)
    classType = checkClass(cont)
    ascii = ""
    data = []
    if classType == "regIOobject":
        [ascii, data] = separateContents(cont)
    elif classType == "labelList":
        [ascii, data] = separateLabelListContents(cont)
    elif (classType == "faceList" or classType == "faceCompactList" or
          classType == "refinementHistory"):
        [ascii, data] = separateFaceListContents(cont)
    elif classType == "vectorField":
        [ascii, data] = separateVectorFieldContents(cont)
    elif classType == "cellSet":
        [ascii, data] = separateLabelListContents(cont)
    elif classType == "faceSet":
        [ascii, data] = separateLabelListContents(cont)
    elif classType == "pointSet":
        [ascii, data] = separateLabelListContents(cont)
    elif classType == "volScalarField":
        [ascii, data] = separateContents(cont)
    elif classType == "volVectorField":
        [ascii, data] = separateContents(cont)
    elif classType == "volTensorField":
        [ascii, data] = separateContents(cont)
    elif classType == "volSymmTensorField":
        [ascii, data] = separateContents(cont)
    elif classType == "surfaceScalarField":
        [ascii, data] = separateContents(cont)
    elif (classType[-4:] == "List" or classType[-4:] == "list"):
        [ascii, data] = separateFaceListContents(cont)
    else:
        [ascii, data] = separateContents(cont)
    return [ascii, data]


#  remakeDataFormat
#    dataのformatを書き換え
def remakeDataFormat(ascii, form, newForm):
    p = getKeywordPointerMem(form, ascii, 0)
    if p >= 0:
        newCont = ascii[:p] + newForm + ascii[p+len(form):]
        ascii = newCont
    return ascii

#
#  writeMeshSeparate
#    分離されたmeshファイルを書き込む
def writeMeshSeparate(fileName, fileCont):
    ascii = fileCont[0]
    data = fileCont[1]
    cont = fuseAsciiData(fileName, ascii, data)
    writeFileFull(fileName, cont)
    return

#
#  readFieldFull
#    fieldデータを全て読み込む（ascii、binaryの区別なく）
def readFieldFull(fileName):
    fileName = checkFileName(fileName)
    if fileName[-3:] == ".gz":
        #gzip形式でopen、read
        f = gzip.open(fileName)
        cont = f.read()
        f.close()
    else:
        #非圧縮でopen、read
        f=open(fileName, "rb")
        cont = f.read()
        f.close()
    return cont

#
#  writeFieldFull
#    fieldデータを全て書き込み（ascii、binaryの区別なく）
def writeFieldFull(fileName, cont):
    fileName = checkFileName(fileName)
    if fileName[-3:] == ".gz":
        #gzip形式でopen、write
        f = gzip.open(fileName, "w")
        f.write(cont)
        f.close()
    else:
        #非圧縮でopen、write
        f=open(fileName, "w")
        f.write(cont)
        f.close()
    return

#  fuseAsciiData
#    asciiとdataを連結する
def fuseAsciiData(fileName, ascii, data):
    obj = checkObject(ascii)
    cont = "".encode("utf-8")           #文字typeを「str」に設定
    p = 0
    n = 0
    i = 0
    while n >= 0:
        #separatorまでskip
        ps = p
        n = ascii[p:].find("\0")
        if n >= 0:
            #separatorまでのasciiを取得
            p += n
            cont += ascii[ps:p].encode("utf-8")
            #次のseparatorまでskip
            p += 1
            pks = p
            n = ascii[p:].find("\0")
            if n >= 0:
                #dataObjectを取得
                p += n
                key = ascii[pks:p]
                #recObj = key.split(".")[0]          #object名
                #dataNo = int(key.split(".")[1])     #dataID
                recObj = ".".join(key.split(".")[:-1])
                dataNo = int(key.split(".")[-1])
                if obj == recObj:
                    #自身のdata取得
                    cont += data[dataNo][1]
                else:
                    #選択したdata取得
                    fileDir = "/".join(fileName.split("/")[:-1])
                    [dummyAscii, newData] = readMeshSeparate(fileDir + "/" + recObj)
                    cont += newData[dataNo][1]
                p += 1
    cont += ascii[p:].encode("utf-8")
    return cont

#  separateContents
#    field内容をasciiとdata部に分ける
def separateContents(cont):
    if checkBinary(cont) == "ascii":
        [ascii, data, error] = separateAsciiContents(cont)
        if error == 1:
            cont = remakeDataFormat(cont, "ascii", "binary")
            [ascii, data, error] = separateBinaryContents(cont)
    else:
        [ascii, data, error] = separateBinaryContents(cont)
        if error == 1:
            cont = remakeDataFormat(cont, "binary", "ascii")
            [ascii, data, error] = separateAsciiContents(cont)
    return [ascii, data]

#  separateLabelListContents
#    labelList形式の内容を分離する
def separateLabelListContents(cont):
    if checkBinary(cont) == "ascii":
        [ascii, data, error] = separateAsciiMultiListContents(cont, "label")
        if error == 1:
            cont = remakeDataFormat(cont, "ascii", "binary")
            [ascii, data, error] = separateBinaryMultiListContents(cont, "label")
    else:
        [ascii, data, error] = separateBinaryMultiListContents(cont, "label")
        if error == 1:
            cont = remakeDataFormat(cont, "binary", "ascii")
            [ascii, data, error] = separateAsciiMultiListContents(cont, "label")
    return [ascii, data]

#  separateFaceListContents
#    faceList形式の内容を分離する
def separateFaceListContents(cont):
    if checkBinary(cont) == "ascii":
        [ascii, data, error] = separateAsciiMultiListContents(cont, "faceList")
        if error == 1:
            cont = remakeDataFormat(cont, "ascii", "binary")
            [ascii, data, error] = separateBinaryMultiListContents(cont, "label")
    else:
        [ascii, data, error] = separateBinaryMultiListContents(cont, "label")
        if error == 1:
            cont = remakeDataFormat(cont, "binary", "ascii")
            [ascii, data, error] = separateAsciiMultiListContents(cont, "faceList")
    return [ascii, data]

#  separateVectorFieldContents
#    vectorField形式の内容を分離する
def separateVectorFieldContents(cont):
    if checkBinary(cont) == "ascii":
        [ascii, data, error] = separateAsciiMultiListContents(cont, "vector")
        if error == 1:
            cont = remakeDataFormat(cont, "ascii", "binary")
            [ascii, data, error] = separateBinaryMultiListContents(cont, "vector")
    else:
        [ascii, data, error] = separateBinaryMultiListContents(cont, "vector")
        if error == 1:
            cont = remakeDataFormat(cont, "binary", "ascii")
            [ascii, data, error] = separateAsciiMultiListContents(cont, "vector")
    return [ascii, data]

#  separateAsciiContents
#    asciiFile内容をasciiとdata部に分ける
def separateAsciiContents(cont):
    error = 0
    obj = checkObject(cont)
    ascii = ""
    data = []
    p = skipFoamFileMem(cont, 0)
    ascii = cont[:p]
    n = 0
    i = 0
    while n>=0 and error == 0:
        ps = p
        n = cont[p:].find(" List<")
        if n >= 0:
            p += n
            [dataType, nData] = getDataTypeNumData(cont, p)
            dataHeader = [dataType, nData]
            n = cont[p:p+100].find("(")
            if n >= 0:
                p += n + 1
                pe = p
                if int(nData) < 100:
                    #データ数が少ない場合
                    [contData, p] = getSmallPairMem(cont, p-1)
                    if contData != "":
                        spId = obj + "." + str(i)
                        ascii += cont[ps:pe] + ("\0" + spId + "\0")
                        data.append([dataHeader, contData])
                        error = confirmAscii(contData)
                        p -= 1
                else:
                    #データ数が多い場合
                    n = cont[p:].find("\n)")
                    if n>=0:
                        spId = obj + "." + str(i)
                        ascii += cont[ps:pe] + ("\0" + spId + "\0")
                        p += n + 1
                        contData = cont[pe:p]
                        data.append([dataHeader, contData])
                        error = confirmAscii(contData)
        i += 1
    ascii += cont[ps:]
    return [ascii, data, error]

#  confirmAscii
#    dataがasciiかどうかチェック
def confirmAscii(cont):
    error = 0
    data = cont[:10]
    for chara in data:
        if chara < chr(0x08) or chara > chr(0x80):
            error = 1
            break
    return error

#  separateBinaryContents
#    binaryFile内容をasciiとdata部に分ける
def separateBinaryContents(cont):
    error = 0
    obj = checkObject(cont)
    data = []
    p = skipFoamFileMem(cont, 0)
    ascii = cont[:p]
    n = 0
    i = 0
    while n>=0 and error == 0:
        ps = p
        n = cont[p:].find(" List<")
        if n >= 0:
            p += n
            [binaryType, nData] = getDataTypeNumData(cont, p)
            dataHeader = [binaryType, nData]
            n = cont[p:p+100].find("(")
            if n >= 0:
                p += n + 1
                pe = p
                [binary, p] = getBinaryData(binaryType, nData, cont, p)
                if len(binary) != 0:
                    spId = obj + "." + str(i)
                    ascii += cont[ps:pe] + ("\0" + spId + "\0")
                    data.append([dataHeader, binary])
                    #errorチェック
                    if cont[p] != ")":
                        error = 1
        i += 1
    ascii += cont[ps:]
    return [ascii, data, error]


#  separateAsciiMultiListContents
#    複数のlist形式のascii内容を分離する
def separateAsciiMultiListContents(cont, dataType):
    error = 0
    [nNum, nByte, valType] = getDataFormat(dataType)
    obj = checkObject(cont)
    data = []
    p = skipFoamFileMem(cont, 0)
    ascii = cont[:p]
    n = 0
    i = 0
    while n >= 0 and error == 0:
        ps = p
        n = cont[p:].find("(")
        if n >= 0:
            pp = p + n
            [nData, p] = getBackKeywordMem(cont, p+n)
            dataHeader = [dataType, nData]
            if int(nData) < 100:
                #データ数少ない場合
                p = pp
                pe = pp + 1
                [dataCont, p] = getSmallPairMem(cont, p)
                ascii += cont[ps:pe] + ("\0" + obj + "." + str(i) + "\0")
                p -= 1
                data.append([dataHeader, dataCont])
                error = confirmAscii(dataCont)
            else:
                #データ数多い場合
                p = pp
                pe = pp + 1
                ascii += cont[ps:pe] + ("\0" + obj + "." + str(i) + "\0")
                ps = p + 1
                n = cont[p:].find("\n)")
                p += n + 1
                dataCont = cont[ps:p]
                data.append([dataHeader, dataCont])
                #ascii += cont[p:]
                error = confirmAscii(dataCont)
        i += 1
    ascii += cont[ps:]
    return [ascii, data, error]

#  separateBinaryMultiListContents
#    複数のlist形式のbinary内容を分離する
def separateBinaryMultiListContents(cont, dataType):
    [ascii, data, error] = separateBinaryFaceListContents(cont, dataType)
    return [ascii, data, error]


#  separateBinaryFaceListContents
#    faceCompactList形式のbinary内容を分離する
def separateBinaryFaceListContents(cont, dataType):
    error = 0
    [nNum, nByte, valType] = getDataFormat(dataType)
    obj = checkObject(cont)
    data = []
    p = skipFoamFileMem(cont, 0)
    ascii = cont[:p]
    n = 0
    i = 0
    while n >= 0 and error == 0:
        ps = p
        n = cont[p:].find("(")
        if n >= 0:
            pss = p + n
            [nData, pp] = getBackKeywordMem(cont, p+n)
            dataHeader = [dataType, nData]
            p = pss + 1
            pe = p
            [binary, p] = getBinaryData(dataType, nData, cont, p)
            if len(binary) != 0:
                spId = obj + "." + str(i)
                ascii += cont[ps:pe] + ("\0" + spId + "\0")
                data.append([dataHeader, binary])
                #errorチェック
                if cont[p] != ")":
                    error = 1
            else:
                ascii += cont[ps:pe]
        i += 1
    ascii += cont[ps:]
    return [ascii, data, error]

#  getDataTypeNumData
#    dataTypeとnDataを取得
def getDataTypeNumData(cont, p):
    block = cont[p:p+100]
    ps = block.find("<") + 1
    pe = block.find(">")
    dataType = block[ps:pe]
    [nData, p] = getKeywordMem(block, pe+1)
    return [dataType, nData]

#  getBinaryData
#    typeに応じてbinaryを取得
def getBinaryData(binaryType, nData, cont, p):
    [nNum, nByte, valType] = getDataFormat(binaryType)
    pe = p + (nNum * int(nData) * nByte)
    dataCont = cont[p:pe]
    return [dataCont, pe]    

#  getDataFormat
#    dataのformatを取得
#    戻り値：[nNum, nByte]
def getDataFormat(dataFormat):
    if dataFormat == "scalar":
        nNum = 1; nByte = 8; valType = "d"
    elif dataFormat == "vector":
        nNum = 3; nByte = 8; valType = "d"
    elif dataFormat == "symmTensor":
        nNum = 6; nByte = 8; valType = "d"
    elif dataFormat == "tensor":
        nNum = 9; nByte = 8; valType = "d"
    elif dataFormat == "label":
        nNum = 1; nByte = 4; valType = "i"
    elif dataFormat == "faceList":
        nNum = 1; nByte = 4; valType = "i"
    elif dataFormat == "bool":
        nNum = 1; nByte = 1; valType = "b"
    else:
        nNum = 0; nByte = 0; valType = ""
    return [nNum, nByte, valType]

#  getOrgFileName
#    非圧縮のfileNameを返す
def getOrgFileName(fileName):
    name = fileName
    if name[-3:] == ".gz":
        name = fileName[:-3]
    return name

#  checkFileName
#    実在するfile名に合わせて、圧縮or非圧縮のfile名に設定する
#    両方存在する場合は、非圧縮のfile名を返す
#    両方共存在しない場合は、そのままfile名を返す
def checkFileName(fileName):
    name = fileName
    if name[-3:] == ".gz":
        name = name[:-3]
    if len(glob.glob(name)) != 0:
        return name
    name += ".gz"
    if len(glob.glob(name)) != 0:
        return name
    return fileName

#  checkBinary
#    データがbinaryかどうかをチェック
#   戻り値：
def checkBinary(cont):
    [foamFile, p] = getKeywordContentsMem("FoamFile", cont, 0)
    lines = foamFile.split(";")
    ans = "ascii"
    for line in lines:
        items = line.split()
        if len(items) >= 2:
            if items[0] == "format":
                ans = items[1]
                break
    return ans

#  checkClass
#    classの読込
def checkClass(cont):
    [foamFile, p] = getKeywordContentsMem("FoamFile", cont, 0)
    lines = foamFile.split(";")
    ans = ""
    for line in lines:
        items = line.split()
        if len(items) >= 2:
            if items[0] == "class":
                ans = items[1]
                break
    return ans

#  checkObject
#    object名の取得
def checkObject(cont):
    [foamFile, p] = getKeywordContentsMem("FoamFile", cont, 0)
    lines = foamFile.split(";")
    ans = ""
    for line in lines:
        items = line.split()
        if len(items) >= 2:
            if items[0] == "object":
                ans = items[1]
                break
    return ans

#  reconstFuseDataAtMaxLinesAB
#    nMaxLinesまで結合されたfuseDataを再構築して全データを取得
def reconstFuseDataAtMaxLinesAB(fileName, cont):
    classType = checkClass(cont)
    #fieldタイプのreconst
    if (classType == "regIOobject" or
        classType == "volScalarField"     or classType == "volVectorField" or
        classType == "volSymmTensorField" or classType == "volTensorField" or
        classType == "surfaceScalarField"):
        allCont = reconstFuseDataAtMaxLinesField(fileName, cont)
    #listタイプのreconst
    elif (classType == "labelList" or classType == "vectorField" or
          classType == "cellSet"   or classType == "faceSet" or
          classType == "pointSet"  or classType == "faceList"):
        allCont = reconstFuseDataAtMaxLinesMultiList(fileName, cont)
    #faceCompactListタイプのreconst
    elif (classType == "faceCompactList" or classType == "refinementHistory"):
        allCont = reconstFuseDataAtMaxLinesMultiList(fileName, cont)
    #Listタイプのreconst（複数のListが存在する場合に設定）
    elif (classType[-4:] == "List" or classType[-4:] == "list"):
        allCont = reconstFuseDataAtMaxLinesMultiList(fileName, cont)
    #その他のreconst（fieldタイプに設定　List<**> **(** **..); ）
    else:
        allCont = reconstFuseDataAtMaxLinesField(fileName, cont)
    return allCont


#  fuseDataをreconstructして元のデータに戻す
#  複数のlist形式に対応
def reconstFuseDataAtMaxLinesMultiList(fileName, cont):
    #separateデータの読込
    [ascii, data] = readFileSeparate(fileName)
    #data部までskipしてdataを入れ替え
    obj = checkObject(cont)
    p = skipFoamFileMem(cont, 0)
    allCont = cont[:p].encode("utf-8")
    n = 0
    while n>=0 and p<len(cont):
        #data部までskip
        ps = p
        n = cont[p:].find("\n(")
        if n >= 0:
            p += n + 2
            pks = p
            allCont += cont[ps:p].encode("utf-8")
            n = cont[pks:].find("\n)")
            p += n + 1
            ple = p
            #keyを取得
            n = cont[pks:p].rfind("...")
            if n >= 0:
                p = pks + n
                n = cont[pks:p].rfind("...")
                p = pks + n
                key = cont[p:ple].split("...")[1]
                #recObj = key.split(".")[0]
                #dataNo = int(key.split(".")[1])
                recObj = ".".join(key.split(".")[:-1])
                dataNo = int(key.split(".")[-1])
                #dataを入れ替え
                if obj == recObj:
                    #自身のdataに入れ替え
                    allCont += data[dataNo][1]
                    p = ple
                else:
                    #他のdataに入れ替え
                    newFile = "/".join(fileName.split("/")[:-1]) + "/" + recObj
                    [newAscii, newData] = readFileSeparate(newFile)
                    allCont += newData[dataNo][1]
                    p = ple
            else:
                #keyが無い場合は、そのまま取得
                allCont += cont[pks:ple].encode("utf-8")
                p = ple
    allCont += cont[p:].encode("utf-8")
    return allCont  

#  fuseDataをreconstructして元のデータに戻す
#  fieldData
def reconstFuseDataAtMaxLinesField(fileName, cont):
    #編集内容の文字コードを「str」に設定
    if type(cont) == unicode:
        cont = cont.encode("utf-8")
    #separateデータを読込
    [ascii, data] = readFileSeparate(fileName)
    #data部までskipしてdataを入れ替え
    obj = checkObject(cont)
    allCont = ""
    n = 0
    p = 0
    while n >= 0 and p < len(cont):
        #data部までskip
        ps = p
        n = cont[p:].find(" List<")
        if n >= 0:
            p += n
            n = cont[p:].find("(")
            if n >= 0:
                p += n +1
                pks = p
                allCont += cont[ps:p]
                n = skipLineEnd(cont[pks:])
                p += n
                ple = p
                #keyを取得
                n = cont[pks:p].rfind("...")
                if n >= 0:
                    p = pks + n
                    n = cont[pks:p].rfind("...")
                    p = pks + n
                    key = cont[p:ple].split("...")[1]
                    #recObj = key.split(".")[0]
                    #dataNo = int(key.split(".")[1])
                    recObj = ".".join(key.split(".")[:-1])
                    dataNo = int(key.split(".")[-1])
                    #dataを入れ替え
                    if obj == recObj:
                        #自身のdataに入れ替え
                        allCont += data[dataNo][1]
                        p = ple
                    else:
                        #他のdataに入れ替え
                        newFile = "/".join(fileName.split("/")[:-1]) + "/" + recObj
                        [newAscii, newData] = readFileSeparate(newFile)
                        allCont += newData[dataNo][1]
                        p = ple  
                else:
                    #keyが無い場合は、そのまま取得
                    allCont += cont[pks:ple]
                    p = ple
    allCont += cont[p:]
    return allCont

#  getFuseDataAtMaxLinesAB
#    ascii binary dataをnMaxLinesまで結合して返す
def getFuseDataAtMaxLinesAB(fileName, ascii, data, nMaxLines):
    obj = checkObject(ascii)
    cont = ""
    i = 0
    p = 0
    n = 0
    while n >= 0:
        #separatorまでskip
        ps = p
        n = ascii[p:].find("\0")
        if n >= 0:
            #separatorまでのasciiを取得
            p += n
            cont += ascii[ps:p]
            #次のseparatorまでskip
            p += 1
            pks = p
            n = ascii[p:].find("\0")
            if n >= 0:
                #dataObjectを取得
                p += n
                key = ascii[pks:p]
                #recObj = key.split(".")[0]
                #dataNo = int(key.split(".")[1])
                recObj = ".".join(key.split(".")[:-1])
                dataNo = int(key.split(".")[-1])
                #dataObjectのdataを取得
                if obj == recObj:
                    #自身のdataをnMaxLinesまで取得
                    sourceData = data[dataNo]
                else:
                    #他のdataを取得
                    fileDir = "/".join(fileName.split("/")[:-1])
                    [newAscii, newData] = readFileSeparate(fileDir + "/" + recObj)
                    sourceData = newData[dataNo]
                #dataをsourceから取得し、set
                if checkBinary(ascii) == "binary":
                    #binary
                    smallData = getDataAtMaxLinesBinary(data[dataNo], nMaxLines)
                    cont += smallData + "..." + key + "...\n"
                else:
                    #ascii
                    smallData = getDataAtMaxLinesAscii(data[dataNo], nMaxLines)
                    if smallData[-3:] == "...":
                        cont += smallData + key + "...\n"
                    else:
                        cont += smallData
                p += 1
    cont += ascii[p:]
    return cont

#  getDataAtMaxLinesAscii
#    ascii data をnMxLinesまで取得
def getDataAtMaxLinesAscii(data, nMaxLines):
    asciiData = data[1]
    if nMaxLines == -1:
        nMaxLines = len(asciiData)
    i = 0
    cont = ""
    p = 0
    while i <= nMaxLines and p<len(asciiData):
        [line , p] = getCRLine(asciiData, p)
        cont += line
        i += 1
    if i > nMaxLines:
        cont += "..."
    return cont

#  getDataAtMaxLinesBinary
#    binary dataをnMxLinesまで取得
def getDataAtMaxLinesBinary(data, nMaxLines):
    [nNum, nByte, valType] = getDataFormat(data[0][0])
    if nMaxLines == -1:
        convByte = len(data[1])
    else:
        convByte = nNum * nByte * nMaxLines

    if len(data[1]) > convByte:
        #nMaxLinesまで変換
        convData = data[1][:convByte]
    else:
        #全データを変換
        convData = data[1]
    n = int(len(convData)/nByte)        #valTypeのデータ数
    numData = struct.unpack(str(n)+valType, convData)
    if nNum == 1:
        stLine = ""; edLine = ""
    else:
        stLine = "("; edLine = ")"
    cont = "\n"
    i = 0
    while i<len(numData):
        cont += stLine
        addCont = ""
        for num in numData[i:i+nNum]:
             addCont += str(num) + " "
        cont += addCont[:-1] + edLine + "\n"
        i += nNum
    return cont

#
#  structBinData
#    binaryDataから数値を取得
def structBinData(data):
    dataType = data[0][0]
    nData = data[0][1]
    binData = data[1]
    #valTypeを取得
    [nNum, nByte, valType] = getDataFormat(dataType)
    #数値変換する数
    num = int(len(binData) / nByte)
    #数値変換
    numData = struct.unpack(str(num)+valType, binData)
    return numData


#  getCRLine
#    1行を取得
def getCRLine(cont, p):
    ps = p
    n = cont[ps:].find("\n")
    if n >= 0:
        p += n + 1
        line = cont[ps:p]
    else:
        line = cont[ps:]
        p = len(cont)
    return [line, p]

#  skipLineEnd
#    行末「);」までskip
def skipLineEnd(asciiCont):
    ne = asciiCont.find(";")
    n = asciiCont[:ne].rfind(")")
    return n

#^^^^^^^^^^^^^^^^^ end of file read write ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^


#
#  getEndTimeFolder
#    endTime, stopAtを取得
#    戻り値:[endTime, stopAt]
def getEndTimeFolder(caseDir):
    wdir = caseDir + "/system"
    if len(glob.glob(wdir + "/controlDict")) == 0:
        return ["", ""]
    else:
        [stopAt, endTime] = controlDict(wdir, ["stopAt", "endTime"])
        return [endTime, stopAt]

#
#  getLatestStartFirstTimeFolder
#    latestTime, startTime, firstTime folderとstartFromを取得
#    戻り値：[latestTime, startTime, firstTime, startFrom]
def getLatestStartFirstTimeFolder(caseDir):
    timeFolders = pyTreeFoam.getTimeFolders(caseDir)
    if len(timeFolders) == 0:
        times = ["", "", "", ""]
    else:
        firstTime = timeFolders[0]
        latestTime = timeFolders[-1]
        wdir = caseDir + "/system"
        #[startFrom, startTime] = controlDict(wdir, ["startFrom", "startTime"])
        ans = controlDict(wdir, ["startFrom", "startTime"])
        if len(ans) == 2:
            [startFrom, startTime] = ans
        else:
            startFrom = ""
            startTime = ""
        times = [latestTime, startTime, firstTime, startFrom]
    return times

#
#  getCurrMeshDir
#    case内のregionからpolyMesh内のfile(boundary等)を探す
#    検索方法は、maxTimeから遡って、検索する
#    region0は、"."で探す
#    戻り値は、caseDirからの相対Dirを返す[例：constant/polyMesh]
def getCurrMeshDir(caseDir, maxTime, region, meshName):
    savePath = os.getcwd()
    os.chdir(caseDir)
    maxTimeNu = float(maxTime)
    timeFlag = 0
    meshDirs = glob.glob("*/" + region + "/polyMesh/" + meshName)
    if len(meshDirs) > 0:
        ansList = []
        for meshDir in meshDirs:
            dirs = meshDir.split("/")
            try:
                a=float(dirs[0])
                if a <= maxTimeNu:
                    ansList.append(dirs[:-1])
            except:
                pass
        if len(ansList) > 0:
            ansList.reverse()
            meshDir = "/".join(ansList[0])
            timeFlag = 1
    if timeFlag == 0:
        meshDir = "constant/" + region + "/polyMesh"
    os.chdir(savePath)
    return meshDir

#
#  getCurrTimeFolder
#    controlDictの内容に従ったtimeFolderを取得
def getCurrTimeFolder(caseDir):
    timeFolder = ""                                 #該当せず
    timeFolders = pyTreeFoam.getTimeFolders(caseDir)
    wdir = caseDir + "/system"
    items = controlDict(wdir, ["startFrom", "startTime"])
    if len(timeFolders) != 0 and items != None:
        if items[0] == "firstTime":
            timeFolder = timeFolders[0]             #firstTime
        elif items[0] == "latestTime":
            timeFolder = timeFolders[-1]            #latestTime
        else:
            if len(items) > 1:
                timeFolder = items[1]
                if len(glob.glob(caseDir + "/" + timeFolder)) == 0:
                    timeFolder = timeFolders[0]     #startTime
            else:
                timeFolder = timeFolders[0]         #startTime無し=firstTime
    return timeFolder


#
#  getSameLocationPatches
#    同じ位置のpatch名を取得
def getSameLocationPatches(polyMeshDirReg0, polyMeshDirReg1):
    minMaxP = []
    minMaxPN = []
    print _(u"patchのmin,max座標を取得中...")
    minMaxP = getPatchesMinMaxLocations(polyMeshDirReg0)
    minMaxPN = getPatchesMinMaxLocations(polyMeshDirReg1)
    samePatch = []
    for patchContP in minMaxP:
        for patchContPN in minMaxPN:
            if checkSameLocationPatch(patchContP, patchContPN)==True:
                samePatch.append([patchContP[0], patchContPN[0]])
                print "   " + patchContP[0] + ", " + patchContPN[0] + _(u" は、同じ位置です")
    return samePatch

#
#  checkSameLocationPatch
#    同じ位置のpatchかどうかチェックする
#    入力：[patchName, [minx, miny, minz], [maxx, maxy, maxz]]
def checkSameLocationPatch(patchCP, patchCPN):
    minPx = patchCP[1][0]
    minPy = patchCP[1][1]
    minPz = patchCP[1][2]
    maxPx = patchCP[2][0]
    maxPy = patchCP[2][1]
    maxPz = patchCP[2][2]
    minPNx = patchCPN[1][0]
    minPNy = patchCPN[1][1]
    minPNz = patchCPN[1][2]
    maxPNx = patchCPN[2][0]
    maxPNy = patchCPN[2][1]
    maxPNz = patchCPN[2][2]
    length = ((maxPx-minPx)**2 + (maxPy-minPy)**2 + (maxPz-minPz)**2)**0.5
    tolerance = abs(length / 100.0)
    if (abs(minPx-minPNx) < tolerance and
        abs(minPy-minPNy) < tolerance and
        abs(minPz-minPNz) < tolerance and
        abs(maxPx-maxPNx) < tolerance and
        abs(maxPy-maxPNy) < tolerance and
        abs(maxPz-maxPNz) < tolerance):
        flag = True
    else:
        flag = False
    return flag

#
#  getPatchesMinMaxLocations
#    patchの最大、最小値を取得して返す
#    戻り値：[[patchName, minP, maxP]]
def getPatchesMinMaxLocations(polyMeshDir):
    patchConts = getPatchCont(polyMeshDir)
    faceDir = polyMeshDir + "/faces"
    f=open(faceDir); faceContents=f.readlines(); f.close()
    pointDir = polyMeshDir + "/points"
    f=open(pointDir); pointContents=f.readlines(); f.close()
    minMaxP=[]
    for patch in patchConts:
        patchName = patch[0]
        nFaces = patch[1]
        startFace = patch[2]
        [minP, maxP] = getPatchMinMaxLocations(
                        faceContents, pointContents, startFace, nFaces)
        minMaxP.append([patchName, minP, maxP])
    return minMaxP

#
#  getPatchCont
#    boundaryからpatch内容を取得
#    戻り値：[[patchName, nFaces, startFace]]
def getPatchCont(polyMeshDir):
    boundDir = polyMeshDir + "/boundary"
    f=open(boundDir); contents=f.read(); f.close()
    p = skipFoamFileMem(contents, 0)
    #最初のpatch名を取得
    flag = 0
    [keyword, p] = getKeywordMem(contents, p)
    while keyword != "" and flag == 0:
        try:
            a=int(keyword)
            [keyword, p] = getKeywordMem(contents, p)
        except:
            flag = 1
    #patch名と内容をセット
    patchName = keyword
    [patchCont, p] = getMiddlePairMem(contents, p)
    ans = []
    while patchCont != "":
        nFaces = 0
        startFace = 0
        lines = patchCont.split("\n")
        for line in lines:
            cont = line.split(";")[0].split()
            if len(cont) >= 1:
                if cont[0] == "nFaces":
                    nFaces = int(cont[1])
                elif cont[0] == "startFace":
                    startFace = int(cont[1])
        patch = [patchName, nFaces, startFace]
        ans.append(patch)
        [[patchName, patchCont], p] = getNextKeywordAndContentsMem(contents, p)
    return ans

#
#  getPatchMinMaxLocation
#    patchのmin,max座標値を取得
def getPatchMinMaxLocations(faceContents, pointContents, startFace, nFaces):
    i=0
    while i<len(faceContents) and faceContents[i][0]!="(":    
        i+=1
    if i<len(faceContents):
        i+=1
        i = i+startFace
        ie = i+nFaces
        faceCont = faceContents[i:ie]
        pointNo = []
        for line in faceCont:
            [facePoints, p] = getSmallPairMem(line, 0)
            pNo = facePoints.split()
            pointNo.append(pNo)
        i=0
        while i<len(pointContents) and pointContents[i][0]!="(":
            i+=1
        if i<len(pointContents):
            i+=1
            pointCont = pointContents[i:]
            p = pointNo[0][0]
            pxyz = pointCont[int(p)]
            [point, p] = getSmallPairMem(pxyz, 0)
            point = point.split()
            minP = [float(point[0]), float(point[1]), float(point[2])]
            maxP = [float(point[0]), float(point[1]), float(point[2])]
            for point in pointNo:
                for p in point:
                    pxyz=pointCont[int(p)]
                    [point3, p] = getSmallPairMem(pxyz, 0)
                    point3 = point3.split()
                    px = float(point3[0])
                    py = float(point3[1])
                    pz = float(point3[2])
                    if minP[0] > px:
                        minP[0] = px
                    if minP[1] > py:
                        minP[1] = py
                    if minP[2] > pz:
                        minP[2] = pz
                    if maxP[0] < px:
                        maxP[0] = px
                    if maxP[1] < py:
                        maxP[1] = py
                    if maxP[2] < pz:
                        maxP[2] = pz
        else:
            print _(u"「points」ファイルから座標が読み取れません。")
            minP=[]
            maxP=[]
    else:
        print _(u"「faces」ファイルからpointNoが読み取れません。")
        minP=[]
        maxP=[]
    return [minP, maxP]


#
#  controlDict
#    conrolDictの内容を取得
def controlDict(wdir, items):
    fileName = wdir + "/controlDict"
    ans = []
    try:
        f = open(fileName)
        contents = f.readlines()
        f.close
    except:
        #print "error: ", wdir, "は、OpenFOAMのCaseDirではありません!!"
        ans = []
        return ans
    i = 0
    contents = deleteComments(contents)
    for item in items:
        ans.append("")
        for line in contents:
            if re.search(item, line) != None:
                a=getValue(item, line)
                if a != "":
                    ans[i] = a
        i = i + 1
    return ans

#
#  getFoamFile
#    foamFileの内容を取得
#    入力     [path, fileName]
#    戻り値　　[[item, contents]]
def getFoamFile(wdir, fileName):
    work = wdir + "/" + fileName
    ans = []
    if glob.glob(work) != "":
        cont = readFile2000Bytes(work)
        [contents, p] = getKeywordContentsMem("FoamFile", cont, 0)
        lines = contents.split(";")
        for line in lines:
            a = line.split()
            if len(a) > 1:
                items = [a[0], a[1]]
                ans.append(items)
    else:
        print "error:",wdir,"/",fileName,_(u" は、存在しません。")
    return ans

#----------------文字列からの検索---------------
#
#  skipFoamFileMem
#    FoamFileをskipする
def skipFoamFileMem(strings, p):
    [dummy, p] = getKeywordContentsMem("FoamFile", strings, p)
    return p

#
#  getC1lineMem
#    「;」までの一行を取得する。
#       eofの場合は、lineCont=""が戻る
def getC1lineMem(strings, p):
    lineCont = ""
    [chara, p] = skipNoCharaMem(strings, p)
    p += 1
    while chara != "" and chara != ";" and p<len(strings):
        if chara == "(":
            p -= 1
            lineCont += "("
            [cont, p] = getSmallPairMem(strings, p)
            lineCont += cont + ")"
        elif chara == "{":
            lineCont += "{"
            [cont, p] = getMiddlePairMem(strings, p-1)
            lineCont += cont + "}"
        else:
            lineCont += chara
        [chara, p] = get1charaMem(strings, p)
    if chara == ";":
        lineCont += chara
    return [lineCont, p]

#
#  replace1lineMem
#    1line分を入れ替える
#    戻り値：[cont, p, keyword]
#    失敗した場合、keywordが「""」になる。
def replace1lineMem(cont, p, newLine):
    [newKeyword, pp] = getKeywordMem(newLine, 0)
    if newKeyword == "":
        return [cont, len(cont), ""]

    [lineCont, p, kind] = get1lineMem(cont, p)
    [keyword, pp] = getKeywordMem(lineCont, 0)
    ps = p - len(lineCont)
    while keyword != newKeyword and lineCont != "":
        [lineCont, p, kind] = get1lineMem(cont, p)
        [keyword, pp] = getKeywordMem(lineCont, 0)
        ps = p - len(lineCont)
    if lineCont == "":
        return [cont, len(cont), ""]

    newCont = cont[:ps] + newLine + cont[p:]
    p = ps + len(newLine)
    return [newCont, p, keyword]

#
#  get1lineMem
#    一行を取得し、行の内容と種類を取得
#    line:「...;」、keyword:「kweyword{}」、include:「# include "xxx.H"」、
#    other:「# xxxx yyyy」を取得
#    eofの場合は、lineCont=""が戻る
#    コメント文もそのまま取得する。
def get1lineMem(strings, p):
    lineCont = ""
    kind = ""
    if p >= len(strings):
        return [lineCont, p, kind]

    [chara, p] = skipNoCharaMem(strings, p)         #ﾎﾟｲﾝﾀを空白以外に設定
    if p >= len(strings) or chara == "":
        return [lineCont, p, kind]

    if chara == ";":
        lineCont = ";"
        p += 1
    elif chara == "#":
        #「#include ...」の取得
        sp = p
        p += 1      #ﾎﾟｲﾝﾀを「#」の次に進める
        [chara, p] = skipNoCharaMem(strings, p)
        [name, p] = getKeywordMem(strings, p)    #「include」取得
        [keyword, p] = getKeywordMem(strings, p)    #「"xxx/H"」取得
        lineCont = strings[sp:p-1]
        #kind = "include"
        if name == "include":
            kind = "include"
        elif name == "includeEtc":
            kind = "includeEtc"
        else:
            kind = "other"
    else:
        sp = p
        [keyword, p] = getKeywordMem(strings, p)
        [chara, p] = skipNoCharaMem(strings, p)
        if chara == "{":
            #「keyword{}」の取得
            p -= 1              #ポインタを戻す
            [cont, p] = getMiddlePairMemComm(strings, p)
            #[cont, p] = getMiddlePairMemF(strings, p)
            #[cont, p] = getMiddlePairMem(strings, p)
            lineCont = strings[sp:p]
            kind = "keyword"
        else:
            #「...;」を取得
            while chara != ";" and chara != "":
                if chara == "(":
                    p -= 1      #ポインタを戻す
                    [dummy, p] = getSmallPairMem(strings, p)
                elif chara == "{":
                    p -= 1      #ポインタを戻す
                    [dummy, p] = getMiddlePairMem(strings, p)
                [chara, p] = get1charaMem(strings, p)
            lineCont = strings[sp:p]
            kind = "line"
    return [lineCont, p, kind]

#
#  getNextKeywordAndContentsMem
#    次のkeywordと{}の内容を取得する
def getNextKeywordAndContentsMem(strings, p):
    flag = 0
    chara = " "
    while flag == 0 and chara != "":
        [keyword, p] = getKeywordMem(strings, p)
        [chara, p] = skipNoCharaMem(strings, p)
        if chara == "{":
            flag = 1
    p -= 1
    [contents, p] = getMiddlePairMem(strings, p)
    return [[keyword, contents], p]

#
#  getKeywordPointerMem
#    指定したkeywordのpointerを取得する（memory上で）
#　　（高速検索している為、keywordがコメント中でも、検索するの注意！）
#    合致しない場合は、戻り値「-1」になる。
def getKeywordPointerMem(keyword, strings, p):
    if p>=0 and p<len(strings):
        #高速検索のため、string.find()を使用してポインタを設定
        flag = 0
        while (p>=0) and (flag==0):
            pp = p
            p = string.find(strings[p:], keyword)
            if p>=0:
                p = pp + p
                #keyword前後の文字をチェック
                if (p-1 < 0):
                    stChara = strings[0]
                else:
                    stChara = strings[p-1]
                pe = p+len(keyword)
                if pe>=len(strings):
                    edChara = "\n"
                else:
                    edChara = strings[p+len(keyword)]
                if ((checkChara(stChara) == False) and
                    (checkChara(edChara) == False)):
                    flag = 1
                else:
                    p+=1
        return p

#
#  getKeywordContentsMem
#    指定したkeywordの内容を取得する（memory上で）
#    （高速検索している為、keywordがコメント中でも、検索するので注意）
#     ({}内データのみ取得する。()内は取得しない
def getKeywordContentsMem(keyword, strings, p):
    if p>=0 and p<len(strings):
        #高速検索のため、string.find()を使用してポインタを設定
        flag = 0
        while (p>=0) and (flag==0):
            pp = p
            p = string.find(strings[p:], keyword)
            if p>=0:
                p = pp + p
                #keyword前後の文字をチェック
                if (p-1 < 0):
                    stChara = strings[0]
                else:
                    stChara = strings[p-1]
                pe = p+len(keyword)
                if pe>=len(strings):
                    edChara = "\n"
                else:
                    edChara = strings[p+len(keyword)]
                if ((checkChara(stChara) == False) and
                    (checkChara(edChara) == False)):
                    flag = 1
                else:
                    p+=1
        #新しいポインタで再度検索し、データ取得
        if p>=0:
            [word, p] = getKeywordMem(strings, p)
            while (word != keyword) and (word != ""):
                [word, p] = getKeywordMem(strings, p)
            [contents, p] = getMiddlePairMemF(strings, p)
        else:
            contents = ""
    else:
        contents = ""
    return [contents, p]

#
#  getBackKeywordMem
#    遡ってkeywordを取得（memory上で）
def getBackKeywordMem(contents, p):
    #文字をbackする
    chara = contents[p]
    while checkChara(chara) == True:
        p -= 1
        chara = contents[p]
    #空白をbackSkipする
    while checkChara(chara) == False:
        p -= 1
        chara = contents[p]
    #文字をbackする
    while checkChara(chara) == True:
        p -= 1
        chara = contents[p]
    #keywordを取得
    [keyword, p] = getKeywordMem(contents, p)
    return [keyword, p]

#
#  getKeywordMem
#    keywordを取得（memory上で）
def getKeywordMem(strings, p):
    #文字まで進む
    [chara, p] = get1charaMem(strings, p)
    while (checkChara(chara) != True) and (chara!=""):
        [chara, p] = get1charaMem(strings, p)
    #keyword取得
    keyWord = ""
    if chara == '"':
        #""で囲まれる場合
        keyWord += chara
        [chara, p] = get1charaMem(strings, p)
        while (chara != '"') and (chara!=""):
            keyWord += chara
            [chara, p] = get1charaMem(strings, p)
        #最後の"をkeywordに追加しポインタを1ヶ進める
        keyWord += chara
        p += 1
    else:
        #通常の文字列
        while (checkChara(chara) == True) and (chara!=""):
            keyWord += chara
            [chara, p] = get1charaMem(strings, p)
    return [keyWord, p]

#
#  getMiddlePairMem
#    { }内データを取得し返す（memory上で）
#    コメント部分は、削除する
def getMiddlePairMem(strings, p):
    #"{"まで進む
    [chara, p] = get1charaMem(strings, p)
    while (chara != "{") and (chara != ""):
        [chara, p] = get1charaMem(strings, p)
    #"}"まで読み込む
    count = 1
    contents = ""
    while (count > 0) and (chara != ""):
        [chara, p] = get1charaMem(strings, p)
        if chara == "{":
            count = count + 1
        elif chara == "}":
            count = count - 1
        contents += chara
    if contents != "":
        if contents[-1] == "}":
            contents = contents[:-1]
    return [contents, p]

#
#  getMiddlePairMemComm
#    { }内データを取得し返す（memory上で）
#    コメント部分をそのまま残す
def getMiddlePairMemComm(strings, p):
    #"{"まで進む
    [chara, p] = get1charaMem(strings, p)
    while (chara != "{") and (chara != ""):
        [chara, p] = get1charaMem(strings, p)
    ps = p + 1
    #"}"まで読み込む
    count = 1
    while (count > 0) and (chara != ""):
        [chara, p] = get1charaMem(strings, p)
        if chara == "{":
            count = count + 1
        elif chara == "}":
            count = count - 1
    pe = p-1
    contents = strings[ps:pe]
    return [contents, p]

#
#  getSmallPairMem
#    ( )内データを取得し返す（memory上で）
def getSmallPairMem(strings, p):
    #"("まで進む
    [chara, p] = get1charaMem(strings, p)
    while (chara != "(") and (chara != ""):
        [chara, p] = get1charaMem(strings, p)
    #")"まで読み込む
    count = 1
    contents = ""
    while (count > 0) and (chara != ""):
        [chara, p] = get1charaMem(strings, p)
        if chara == "(":
            count = count + 1
        elif chara == ")":
            count = count - 1
        contents += chara
    if contents != "":
        if contents[-1] == ")":
            contents = contents[:-1]
    return [contents, p]

#
#  getSmallPairMemComm
#    ( )内データを取得し返す（memory上で）
#    コメント文もそのまま取得する。
def getSmallPairMemComm(strings, p):
    #"("まで進む
    [chara, p] = get1charaMem(strings, p)
    while (chara != "(") and (chara != ""):
        [chara, p] = get1charaMem(strings, p)
    ps = p
    #")"まで読み込む
    count = 1
    while (count > 0) and (chara != ""):
        [chara, p] = get1charaMem(strings, p)
        if chara == "(":
            count = count + 1
        elif chara == ")":
            count = count - 1
    pe = p - 1
    contents = strings[ps:pe]
    return [contents, p]

#
#  SpNoCharaMem
#    空白以外の文字までskipする
#    ﾎﾟｲﾝﾀは空白以外の文字位置を返す。charaはﾎﾟｲﾝﾀが示す文字を返す
def skipNoCharaMem(strings, p):
    if p>=len(strings):
        return ["", p]

    [chara, p] = get1charaMem(strings, p)
    while (chara == " " or chara == "\n" or chara == "\t"):
        [chara, p] = get1charaMem(strings, p)
    p -= 1
    return [chara, p]

#
#  get1charaMem
#    get 1 character on memory
#    文字列（strings）とポインタ（p）を設定
def get1charaMem(strings, p):
    if p>=len(strings):
        return ["", p]

    [chara, p] = get1c(strings, p)
    if chara == "/":
        [nextChara, p] = get1c(strings, p)
        if nextChara == "/":
            p = skipUntilCRMem(strings, p)      #行末までスキップ
            [chara, p] = get1c(strings, p)
        elif nextChara == "*":
            p= skipUntilSSMem(strings, p)       #"*/"までスキップ
            [chara, p] = get1c(strings, p)
        else:
            p-=1                                #1文字戻す
    return [chara, p]

#行末までスキップ
def skipUntilCRMem(strings, p):
    if p>=0 and p<len(strings):
        [chara, p] = get1c(strings, p)
        while (chara != "\n") and (chara != ""):
            [chara, p] = get1c(strings, p)
            #print chara," ",
        if chara == "\n":
            p-=1                                        #1文字戻す
    else:
        p=len(strings)
    return p
#*/までスキップ
def skipUntilSSMem(strings, p):
    if p>=0 and p<len(strings)-1:
        charas = strings[p:p+2]; p+=1
        while (charas != "*/") and (p<len(strings)-1):
            charas = strings[p:p+2]; p+=1
            #print charas, " ",
        if charas == "*/":
            p+=1
        else:
            p=len(strings)
    else:
        p=len(strings)
    return p
#spスキップ
def skipSpaceMem(strings, p):
    if p>=0 and p<len(strings)-1:
        while strings[p] == " ":
            p+=1
    else:
        p=len(strings)
    return p
#1文字取得
def get1c(strings, p):
    #if p>=0 and p<len(strings):
    if p>=0 and p<len(strings)-1:
        chara = strings[p]; p +=1
    else:
        chara = ""; p = len(strings)
    return [chara, p]

#
#  checkChara
#    keywordとして使える文字のチェック
def checkChara(chara):
    if chara > " " and chara <= "z":
        if chara == "{" or chara == "}":
            flag = False
        elif chara == "(" or chara == ")":
            flag = False
        elif chara == ";":
            flag = False
        else:
            flag = True
    else:
        flag = False
    return flag


#
#  getPatchNameFromBoundary
#    boundaryがあるDirから取得
def getPatchNameFromBoundary(wdir):
    fileName = wdir + "/boundary"
    ans = PatchNameFromBoundary_sub(fileName)
    return ans

#
#  PatchNameFromBoundary
#    polyMeshがあるDirから取得
#    戻り値：[[name, type, nFaces, startFace], ...]
def PatchNameFromBoundary(polyMeshDir):
    fileName = polyMeshDir + "/polyMesh/boundary"
    ans = PatchNameFromBoundary_sub(fileName)
    return ans

def PatchNameFromBoundary_sub(fileName):
    ans = []
    try:
        f = open(fileName)
        contents = f.read()
        f.close
    except:
        print "error: ", fileName, _(u"は、polyMeshのDirでは、ありません。")
        ans = []
        return ans
    ans = []
    p=skipFoamFileMem(contents, 0)
    [values, p] = getSmallPairMem(contents, p)
    [[keyword, value], p] = getNextKeywordAndContentsMem(values, 0)
    while keyword != "":
        name = ["", "", "", ""]
        name[0] = keyword
        lines = value.split("\n")
        for line in lines:
            items = line.split()
            if len(items)>1:
                if items[0] == "type":
                    name[1] = items[1]
                elif items[0] == "nFaces":
                    name[2] = items[1]
                elif items[0] == "startFace":
                    name[3] = items[1]
        ans.append(name)
        [[keyword, value], p] = getNextKeywordAndContentsMem(values, p)
    return ans

#------------ cellZone, faceZoneの取得（新）---------------

#  cellZoneNamesDir
#    polyMeshDirを指定してcellZoneNamesを取得
def cellZoneNamesDir(polyMeshDir):
    fileName = polyMeshDir + "/cellZones"
    zones = getZoneNamesF_sub(fileName)
    return zones

#  cellZoneNamesF
#    cellZone名を取得(コメント文が有っても可）
def cellZoneNamesF(caseDir):
    fileName = caseDir + "/constant/polyMesh/cellZones"
    zones = getZoneNamesF_sub(fileName)
    return zones

#  cellZoneNames
#    currDirからcellZone名を取得
def cellZoneNames():
    caseDir = os.getcwd()
    zones = cellZoneNamesF(caseDir)

#  pointZoneNamesDir
#    polyMeshDirを指定してpointZoneNamesを取得
def pointZoneNamesDir(polyMeshDir):
    fileName = polyMeshDir + "/pointZones"
    zones = getZoneNamesF_sub(fileName)
    return zones

#  faceZoneNamesDir
#    polyMeshDirを指定してfaceZoneNamesを取得
def faceZoneNamesDir(polyMeshDir):
    fileName = polyMeshDir + "/faceZones"
    zones = getZoneNamesF_sub(fileName)
    return zones

#  faceZoneNamesF
#    faceZone名を取得
def faceZoneNamesF(caseDir):
    fileName = caseDir + "/constant/polyMesh/faceZones"
    zones = getZoneNamesF_sub(fileName)
    return zones

#  faceZoneNames
#    currDirからfaceZone名を取得
def faceZoneNames():
    caseDir = os.getcwd()
    zones = faceZoneNamesF(caseDir)
    return zones

#  getZoneNamesF_sub
def getZoneNamesF_sub(fileName):
    #fileが無ければ、直ぐに戻る
    if len(glob.glob(fileName)) == 0:
        fileName += ".gz"
        if len(glob.glob(fileName)) == 0:
            return []
    [contents, data] = readMeshSeparate(fileName)
    #foamFileをスキップ
    p = skipFoamFileMem(contents, 0)
    #コメント文を削除
    contents = deleteCommentChars(contents[p:])
    #zone名の取得
    p = 0
    zones = []
    [keyword, p] = getKeywordMem(contents, p)
    try:
        a=int(keyword)
    except:
        zones.append(keyword)
        [dummy, p] = getMiddlePairMemF(contents, p)
    while keyword != "" and p<len(contents):
        [keyword, p] = getKeywordMem(contents, p)
        if keyword != "":
            zones.append(keyword)
            [dummy, p] = getMiddlePairMemF(contents, p)
    zones.sort()
    return zones

#  getMiddlePairMemF
#    {}内のデータとポインタ取得
#    高速検索用。コメント中に「{}」があると、引っかかる。
def getMiddlePairMemF(contents, p):
    pair = ""
    while pair != "{" and p<len(contents):
        [pair, p] = getNextMiddlePairMemF(contents, p)
    p += 1
    ap = p
    count = 1
    while count > 0 and ap<len(contents):
        [pair, ap] = getNextMiddlePairMemF(contents, ap)
        if pair == "{":
            count += 1
        elif pair == "}":
            count -= 1
    #return [contents[p:ap-1], ap+1]
    return [contents[p:ap-1], ap]

#  getNextMiddlePairMemF
def getNextMiddlePairMemF(contents, p):
    lp = contents[p:].find("{")
    if lp<0:
        lp = len(contents)
    rp = contents[p:].find("}")
    if rp<0:
        rp = len(contents)
    if lp < rp:
        ans = "{"
        ap = p + lp + 1
    elif rp < lp:
        ans = "}"
        ap = p + rp + 1
    else:
        ans = ""
        ap = len(contents)
    return [ans, ap]

#  getSmallPairMemF
#    ()内のデータとポインタ取得
def getSmallPairMemF(contents, p):
    pair = ""
    while pair != "(" and p<len(contents):
        [pair, p] = getNextSmallPairMemF(contents, p)
    p += 1
    ap = p
    count = 1
    while count > 0 and ap<len(contents):
        [pair, ap] = getNextSmallPairMemF(contents, ap)
        if pair == "(":
            count += 1
        elif pair == ")":
            count -= 1
    return [contents[p-1:ap-1], ap]

#  getNextSmallPairMemF
def getNextSmallPairMemF(contents, p):
    lp = contents[p:].find("(")
    if lp<0:
        lp = len(contents)
    rp = contents[p:].find(")")
    if rp<0:
        rp = len(contents)
    if lp < rp:
        ans = "("
        ap = p + lp + 1
    elif rp < lp:
        ans = ")"
        ap = p + rp + 1
    else:
        ans = ""
        ap = len(contents)
    return [ans, ap]

#  deleteCommentsNullLines
#    コメント、空文の削除
def deleteCommentsNullLines(contents):
    contents = deleteCommentChars(contents)
    contents = deleteNullLines(contents)
    return contents

#  deleteNullLines
#    空文の削除
def deleteNullLines(contents):
    lines = contents.split("\n")
    newCont = ""
    for line in lines:
        if len(line.split()) != 0:
            newCont += line + "\n"
    return newCont

#  deleteCommentChars
#    commentの削除
def deleteCommentChars(contents):
    nocomm = ""
    pp=0
    st = 0
    flag = 0
    while pp != -1 and flag == 0:
        pst = contents[st:].find("/*")
        pss = contents[st:].find("//")
        if pst == -1 and pss == -1:
            #comm無し、終了
            flag = 1
        elif pst == -1 and pss != -1:
            #//をスキップ
            commS = pss + st
            commE = skipComm(contents[st:], "\n", pss) + st
        elif pst != -1 and pss == -1:
            #/*をスキップ
            commS = pst + st
            commE = skipComm(contents[st:], "*/", pst) + st 
        elif pst < pss:
            #/*をスキップ
            commS = pst + st
            commE = skipComm(contents[st:], "*/", pst) + st
        else:
            #//をスキップ
            commS = pss + st
            commE = skipComm(contents[st:], "\n", pss) + st

        if flag == 0:
            nocomm += contents[st:commS]
            st = commE + 1
        else:
            nocomm += contents[st:]
    return nocomm

#  skipComm
def skipComm(cont, echar, st):
    p = cont[st:].find(echar)
    if p==-1:
        p = len(cont[st:]) + st
    else:
        p = p + len(echar) + st
        if echar == "\n":
            p -= 2
    return p
#^^^^^^^^^^^^^^^ cellZone, faceZoneの取得（新) end ^^^^^^^^^^^^^^


def deleteComments(contents):
    ans = []
    n = 0; c = 0;ansL = ""
    while n < len(contents):
        if len(contents[n]) == 1:
            c = 0; n = n + 1
        else:
            char = contents[n][c:c+2]
            if char[0] == '"':
                # skip next '"'
                ansL = ansL + char[0]
                c = c + 1
                while contents[n][c] != '"' and c < len(contents[n]):
                    ansL = ansL + contents[n][c]
                    c = c + 1
                ansL = ansL + contents[n][c]
                c = c + 1
            elif char == "/*":
                # skip next "*/"
                while contents[n][c:c+2] != "*/" and n < len(contents):
                    c = c + 1
                    if c >= (len(contents[n]) -1):
                        c = 0; n = n + 1
                c = c + 2
            elif char == "//":
                # skip EOL
                c = 0; n = n +1
            else:
                ansL = ansL + contents[n][c]
                c = c + 1
        if n < len(contents):
            if c >= len(contents[n]) -1:
                c = 0; n = n + 1
                ans.append(ansL); ansL = ""
    comp = []
    # 空文を削除
    for cont in ans:
        if cont != "":
            comp.append(cont)
    # 空白を詰める
    n=0
    for cont in comp:
        if len(cont) > 2:
            i=0; line = ""
            while i < len(cont)-1:
                if cont[i:i+2] != "  ":
                    line=line + cont[i]
                i=i+1
            line = line + cont[i]
            if line[0] == " ":
                comp[n] = line[1:]
            else:
                comp[n] = line
        n=n+1
    return comp


def getValue(item, line):
    a=line.split()
    if a[0] == item:
        if a[1][-1] == ";":
            return a[1][:-1]
        else:
            return a[1]
    else:
        return ""


#
#  test--------------------------------------
import string
if __name__ == "__main__":
    wdir = os.getcwd()
    polyMeshDir0 = "/home/caeuser/CAE/CAE-FOAM/conjugate/conjugateTestModel/conjugateTestCase-DyMesh/subCase/regCase/constant/polyMesh"
    polyMeshDir1 = "/home/caeuser/CAE/CAE-FOAM/conjugate/conjugateTestModel/conjugateTestCase-DyMesh/subCase/regCase/constant/solid2/polyMesh"
    samePatches = getSameLocationPatches(polyMeshDir0, polyMeshDir1)
    print samePatches


